#include "stdafx.h"
#include "WeightedAverage.h"


struct WeightedClip
{
	PClip clip;
	double weight;
		WeightedClip(PClip _clip, double _weight)
		: clip(_clip), weight(_weight) {}
};

class Average : public GenericVideoFilter 
{   
	vector<WeightedClip> clips;
public:
	Average(PClip _child, vector<WeightedClip> _clips, IScriptEnvironment *env);
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
};

Average::Average(PClip _child, vector<WeightedClip> _clips, IScriptEnvironment *env)
	: GenericVideoFilter(_child), clips(_clips)
{
	//if (!vi.IsYV12()) 
	//	env->ThrowError("Average requires YV12.");
}

PVideoFrame __stdcall Average::GetFrame(int n, IScriptEnvironment *env) 
{
	vector<WeightedFrame> data;
		data.reserve(clips.size());
	for (int i=0; i < clips.size(); ++i)
		data.push_back(WeightedFrame(clips[i].clip->GetFrame(n, env), clips[i].weight));

	return WeightedAverage(data, env, vi);
}



AVSValue __cdecl Create_Average(AVSValue arguments, void *user_data, IScriptEnvironment *env) 
{
	AVSValue array = arguments[0];
	int noarguments = array.ArraySize();
	if(noarguments % 2 != 0)
		env->ThrowError("Average requires an even number of arguments.");

	vector<WeightedClip> clips;
	for (int i=0; i < noarguments; i+=2)
		clips.push_back(WeightedClip(array[i].AsClip(), array[i+1].AsFloat()));

	return new Average(clips [0]. clip, clips, env);  
}

extern "C" __declspec(dllexport) const char *__stdcall AvisynthPluginInit2(IScriptEnvironment *env) 
{
	env->AddFunction("Average", ".*", Create_Average, 0);
	return "Average plugin";
}
