#include "stdafx.h"

#include "Weightedaverage.h"
//#include "Cycles.h"
//ofstream logw("logw.txt");
//ofstream timing("timing.txt");

WeightedAverage::WeightedAverage(const vector<WeightedFrame> &data, IScriptEnvironment* env, VideoInfo vi)
 : length(0), source(0), scaledweight(0), result(env->NewVideoFrame(vi, 32))
{
	source.reserve(data.size());
	scaledweight.reserve(data.size());

	const double threshold = 1/512.0;

	for (int i=0; i < data.size(); ++i)
		if ( data[i].weight > threshold || data[i].weight < -threshold )
		{
			++length;
			source.push_back(data[i].frame);
			scaledweight.push_back(int (data[i].weight*scale+0.5));
		}

	if (length == 0)
	return ;//FIX	env->ThrowError("RestoreFPS: no frames to average (after trimming).");
	if (length > 256)
		env->ThrowError("RestoreFPS: Cannot average more than 256 frames.");
	//if (source[0]->GetRowSize() != vi.width) 
	//	env->ThrowError("Destination width does not match clip width.");
	if (source[0]->GetHeight() != vi.height) 
		env->ThrowError("Destination height does not match clip height.");

	bool optimisable = ((env->GetCPUFlags() & CPUF_INTEGER_SSE) != 0);
	if (length > 32) optimisable = false;
	for (int i=0;i<length;++i)
		if ( scaledweight[i] != short (scaledweight[i]) )
			optimisable = false;

	//for (int i=0;i<length;++i) logw <<scaledweight[i]/double(scale)<<"\t";logw << "\n";
	//timing<<length <<"\t"; //for (int i=0;i<;++i) log
	if (length %2== 1)
	{
		source.push_back(data[0].frame);
		scaledweight.push_back(0);
		++length;
	}

//	MeasureCycles cycles;
	if (optimisable && length %2== 0)
	{
	//	cycles.mark();
		averageplane_P3(PLANAR_Y, env);
		//if (length== 16) timing <<double (cycles.mark())/(vi.width*vi.height) << "\n";
		averageplane_P3(PLANAR_U, env);
		averageplane_P3(PLANAR_V, env);
	}
	else
	{
	//	cycles.mark();
		averageplane(PLANAR_Y, env);
		//timing << "fallback\t" <<double (cycles.mark())/(vi.width*vi.height) << "\n";
		averageplane(PLANAR_U, env);
		averageplane(PLANAR_V, env);
	}
}

void WeightedAverage::averageplane(int plane, IScriptEnvironment* env)
{
	vector<const unsigned char*> sourcep(length);
	for (int i=0;i<length;++i)
		sourcep[i] = source[i]->GetReadPtr(plane);
	unsigned char* resultp = result->GetWritePtr(plane);

	const int width = source[0]->GetRowSize(plane);
	const int height = source[0]->GetHeight(plane);

	vector<const int> source_pitch(length);
	for (int i=0;i<length;++i)
		source_pitch[i] = source[i]->GetPitch(plane);//,logw<<source_pitch[i] <<"\t"
	const int result_pitch = result->GetPitch(plane);

	for (int y = 0; y < height; ++y)
	{
		for (int x = 0; x < width; ++x)
		{
			int sum = scale/2;
			for (int i=0;i<length;++i)
				sum += scaledweight[i] *sourcep[i][x];
			resultp[x] =max( min (sum/scale,255),0);
		}
		for (int i=0;i<length;++i)
			sourcep[i] += source_pitch[i];
		resultp += result_pitch;
	}
}
/*
void WeightedAverage::averageplane_P4(int plane, IScriptEnvironment* env)
{
	if (length > 32) 
		env->ThrowError("Optimised WeightedAverage can only process 32 frames.");
	if (length%2==1)
		env->ThrowError("Optimised WeightedAverage requires even number of frames.");
	for (int i=0;i<length;++i)
		if ( scaledweight[i] != short (scaledweight[i]) )
			env->ThrowError("Optimised WeightedAverage weight coefficients too large.");

	_declspec(align(16)) const unsigned char* sourcep[64];
     for (int i=0;i<length;++i)
               sourcep[i] = source[i]->GetReadPtr(plane);
	//for (int i=0;i<length/2;++i)
	//{
	//	sourcep[4*i] = source[2*i]->GetReadPtr(plane);
	//	sourcep[4*i+1] = source[2*i+1]->GetReadPtr(plane);
	//}
	unsigned char* resultp = result->GetWritePtr(plane);

	const int width = source[0]->GetRowSize(plane);
	const int height = source[0]->GetHeight(plane);

	 int source_pitch[32];
	for (int i=0;i<length;++i)
		source_pitch[i] = source[i]->GetPitch(plane);
	const int result_pitch = result->GetPitch(plane);

	_declspec(align(16)) int rounder[4];
	for (int j=0;j<4;++j)
		rounder[j] = scale/2;

	_declspec(align(16)) short multiplier[16*8];
	for (int i=0;i<length/2;++i)
		for (int j=0;j<8;j+=2)
		{
			multiplier[i*8 + j]      = short (scaledweight[2*i]);
			multiplier[i*8 + j+1]   = short (scaledweight[2*i+1]);
		}

	//const int length8 = length*8;
	       const int length4 = length*4;
const int eightwidth = (width% 8 == 0) ? width : (width + 8 - (width% 8));

	for (int y = 0; y < height; ++y)
	{
		 __asm
		 {
			  mov     ebx, [eightwidth]//
			  mov     edi, [resultp]//
			  pxor    xmm7, xmm7
spatial:
			  movdqa  xmm4, [rounder]
			  movdqa  xmm5, [rounder]

			  mov     eax, [length4]//
temporal:
			  mov     ecx, [sourcep+eax-8]
			  mov     edx, [sourcep+eax-4]
			  movq    xmm0, mmword ptr [ecx+ebx-8]    //ecx points to frame n
			  movq    xmm1, mmword ptr [edx+ebx-8]    //edx points to frame n+1

			  movdqa  xmm3, [multiplier+2*eax-16]     //multiplier
			  punpcklbw xmm0, xmm1
			  pshufd  xmm1, xmm0, 0x4e
			  punpcklbw xmm0, xmm7
			  punpcklbw xmm1, xmm7

			  pmaddwd xmm0, xmm3
			  pmaddwd xmm1, xmm3
			sub     eax, 8
			  paddd   xmm4, xmm0
			  paddd   xmm5, xmm1

			jnz      temporal
			  //now have eight pixels worth of result in DWORDs of xmm5:xmm4

			  psrad   xmm4, 8+6
			  psrad   xmm5, 8+6
			  packssdw xmm4, xmm5
			  pmaxsw  xmm4, xmm7//necessarily*** open?
			  //now have eight pixels worth of result in WORDs of xmm4

			  packuswb xmm4, xmm7
			  movq    mmword ptr  [edi+ebx-8], xmm4

			  sub     ebx, 8
			  jnz      spatial

		 }
		 for (int i=0;i<length;++i)
			  sourcep[i] += source_pitch[i];
		 resultp += result_pitch;
	}
}*/
